{$A+,B-,D-,E+,F-,G+,I-,L-,N+,O-,P+,Q-,R-,S-,T-,V-,X+}
{$M 16384,0,655360}
Unit LoKeyb;

interface

const
  evNothing = $00;
  evKeyDown = $02;
  evKeyAuto = $03;
  evKeyUp   = $04;

const
  ksSlowest = $031F;
  ksFastest = $0000;
  ksDefault = $010C;

const
  klScrollLock = $01;
  klNumLock    = $02;
  klCapsLock   = $04;

const
  kbEsc       = 001;
  kbF1        = 059;
  kbF2        = 060;
  kbF3        = 061;
  kbF4        = 062;
  kbF5        = 063;
  kbF6        = 064;
  kbF7        = 065;
  kbF8        = 066;
  kbF9        = 067;
  kbF10       = 068;
  kbF11       = 087;
  kbF12       = 088;
  kbScrlLock  = 070;
  kbTilde     = 041;
  kb1         = 002;
  kb2         = 003;
  kb3         = 004;
  kb4         = 005;
  kb5         = 006;
  kb6         = 007;
  kb7         = 008;
  kb8         = 009;
  kb9         = 010;
  kb0         = 011;
  kbMinus     = 012;
  kbEqual     = 013;
  kbBackSpace = 014;
  kbTab       = 015;
  kbQ         = 016;
  kbW         = 017;
  kbE         = 018;
  kbR         = 019;
  kbT         = 020;
  kbY         = 021;
  kbU         = 022;
  kbI         = 023;
  kbO         = 024;
  kbP         = 025;
  kbOpenSq    = 026;
  kbCloseSq   = 027;
  kbEnter     = 028;
  kbCapsLock  = 058;
  kbA         = 030;
  kbS         = 031;
  kbD         = 032;
  kbF         = 033;
  kbG         = 034;
  kbH         = 035;
  kbJ         = 036;
  kbK         = 037;
  kbL         = 038;
  kbSemicolon = 039;
  kbQoute     = 040;
  kbLShift    = 042;
  kbBackSlash = 043;
  kbZ         = 044;
  kbX         = 045;
  kbC         = 046;
  kbV         = 047;
  kbB         = 048;
  kbN         = 049;
  kbM         = 050;
  kbComma     = 051;
  kbPeriod    = 052;
  kbSlash     = 053;
  kbRShift    = 054;
  kbCtrl      = 029;
  kbAlt       = 056;
  kbSpace     = 057;
  kbHome      = 071;
  kbUp        = 072;
  kbPgUp      = 073;
  kbLeft      = 075;
  kbKey5      = 076;
  kbRight     = 077;
  kbEnd       = 079;
  kbDown      = 080;
  kbPgDn      = 081;
  kbNumLock   = 069;
  kbGrayMinus = 074;
  kbGrayPlus  = 078;
  kbAsterisk  = 055;
  kbIns       = 082;
  kbDel       = 083;

var
  KeybState    : array [0..127] of Boolean;
  LastPress,
  LastRelease,
  LastScanCode : Byte;
  AutoFlag     : Boolean;

var
  KeyboardReady : Boolean;

type
  TEvent = record
             What          : Byte;
             Param1,Param2 : LongInt
           end;
  TKeyEvent = record
                What : Byte;
                Key  : Byte
              end;

procedure InitKeyboard;
procedure DoneKeyboard;
procedure GetKeyEvent (var Event);
procedure Pause;
procedure ClearKeyEvent;
function  CheckKey (Key: Byte): Boolean;
function  CheckDoubleKey (Keys: Word): Boolean;
procedure SetKeybLights (Mask: Byte);
procedure SetKeybSpeed (Speed: Word);
implementation

var
  OldExitProc : procedure;
  i           : Integer;
  OldHandler  : Pointer;

procedure KeybHandler; interrupt; assembler;
asm
  in   al,60h
  mov  LastScanCode,al
  test al,80h
  jnz  @@1
  mov  LastPress,al
  xor  ah,ah
  mov  bx,ax
  mov  AutoFlag,01
  mov  al,[byte ptr KeybState+bx]
  test al,01
  jnz  @@2
  mov  AutoFlag,ah
  mov  [byte ptr KeybState+bx],01
  jmp  @@2
@@1:
  and  al,7Fh
  mov  LastRelease,al
  mov  AutoFlag,00
  xor  ah,ah
  mov  bx,ax
  mov  [byte ptr KeybState+bx],ah
@@2:
  in   al,61h
  mov  ah,al
  or   al,80h
  out  61h,al
  xchg ah,al
  out  61h,al
  mov  al,20h
  out  20h,al
end;

procedure InitKeyboard;
begin
   if not KeyboardReady then
   begin
     asm cli end;
     KeyboardReady:=True;
     OldHandler:=Pointer(MemL[$0000:$09*4]);
     Pointer(MemL[$0000:$09*4]):=@KeybHandler;
     LastPress   :=0;
     LastRelease :=0;
     LastScanCode:=0;
     AutoFlag    :=False;
     for i:=1 to 127 do
       KeybState[i]:=False;
     asm sti end;
   end;
end;

procedure DoneKeyboard;
begin
  if KeyboardReady then
  begin
    asm cli end;
    KeyboardReady:=False;
    Pointer(MemL[$0000:$09*4]):=OldHandler;
    asm sti end;
  end;
end;

procedure GetKeyEvent;
begin
  with TKeyEvent(Event) do
  begin
    Key:=LastScanCode and $7F;
    if LastScanCode=0 then What:=evNothing;
    if LastScanCode<$80 then
      if AutoFlag then
        What:=evKeyAuto
      else
        What:=evKeyDown
    else
      What:=evKeyUp;
    if LastScanCode=0 then What:=evNothing;
    LastScanCode:=0;
    AutoFlag:=False;
  end;
end;

procedure Pause;
var
  T : Boolean;
begin
  T:=KeyboardReady;
  InitKeyboard;
  while LastPress=0 do;
  if not T then DoneKeyboard;
end;

procedure ClearKeyEvent;
begin
  LastScanCode:=0;
end;

function CheckKey;
begin
  CheckKey:=KeybState[Key];
end;

function CheckDoubleKey;
begin
  CheckDoubleKey:=(KeybState[Keys and $7F]) and (KeybState[Keys shr 8]);
end;

procedure SetKeybLights; assembler;
asm
  cli
  mov  al,0EDh
  out  60h,al
  mov  cx,$FFFF
@@1:
  loop @@1
  mov  al,Mask
  out  60h,al
  sti
end;

procedure SetKeybSpeed; assembler;
asm
  cli
  mov ax,0305h
  mov bx,Speed
  int 16h
  sti
end;

procedure NewExitProc; far;
begin
  ExitProc:=@OldExitProc;
  DoneKeyboard;
end;

begin
  KeyboardReady:=False;
  @OldExitProc:=ExitProc;
  ExitProc:=@NewExitProc;
end.